@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemComplexPlugin.cpp 4049 2007-02-18 22:37:10Z chpe $
+ * $Id: totemComplexPlugin.cpp 4293 2007-05-10 14:23:07Z hadess $
  */
 
 #include <mozilla-config.h>
@@ -49,11 +49,10 @@ static const nsCID kClassID =
 static const char kClassDescription[] = "totemComplexPlugin";
 static const char kPluginDescription[] = "Helix DNA Plugin: RealPlayer G2 Plug-In Compatible (compatible; Totem)";
 
+#define TOTEM_COMPLEX_VERSION_BUILD "10.0"
+
 static const totemPluginMimeEntry kMimeTypes[] = {
-	{ "audio/x-pn-realaudio-plugin", "rpm", "audio/vnd.rn-realaudio", FALSE },
-	{ "audio/x-pn-realaudio", "rpm" , NULL, TRUE },
-	{ "application/vnd.rn-realmedia", "rpm", NULL, TRUE },
-	{ "application/smil", "smil", NULL, TRUE }
+	{ "audio/x-pn-realaudio-plugin", "rpm", "audio/vnd.rn-realaudio" },
 };
 
 void*
@@ -1974,8 +1973,7 @@ totemScriptablePlugin::GetUserCountryID(PRInt32 *_retval)
 NS_IMETHODIMP
 totemScriptablePlugin::GetVersionInfo(nsACString & _retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED ();
-  _retval.Assign ("1.0");
+  _retval.Assign (TOTEM_COMPLEX_VERSION_BUILD);
   return NS_OK;
 }
 
