@@ -29,13 +29,14 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "totem-fullscreen.h"
 #include "totem-interface.h"
 #include "totem-time-label.h"
 #include "bacon-video-widget.h"
 
-#define FULLSCREEN_POPUP_TIMEOUT 5 * 1000
+#define FULLSCREEN_POPUP_TIMEOUT 5
 
 static GObjectClass *parent_class = NULL;
 
@@ -60,7 +61,6 @@ struct TotemFullscreenPrivate {
 
 	/* Locks for keeping the popups during adjustments */
 	gboolean          seek_lock;
-	gboolean          vol_lock;
 
 	guint             popup_timeout;
 	gboolean          popup_in_progress;
@@ -161,24 +161,6 @@ totem_fullscreen_window_unrealize_cb (GtkWidget *widget, TotemFullscreen *fs)
 					      G_CALLBACK (totem_fullscreen_theme_changed_cb), fs);
 }
 
-gboolean
-totem_fullscreen_vol_slider_pressed_cb (GtkWidget *widget,
-					GdkEventButton *event,
-					TotemFullscreen *fs)
-{
-	fs->priv->vol_lock = TRUE;
-	return FALSE;
-}
-
-gboolean
-totem_fullscreen_vol_slider_released_cb (GtkWidget *widget,
-					 GdkEventButton *event,
-					 TotemFullscreen *fs)
-{
-	fs->priv->vol_lock = FALSE;
-	return FALSE;
-}
-
 gboolean
 totem_fullscreen_seek_slider_pressed_cb (GtkWidget *widget,
 					 GdkEventButton *event,
@@ -200,8 +182,8 @@ totem_fullscreen_seek_slider_released_cb (GtkWidget *widget,
 static void
 totem_fullscreen_popup_timeout_add (TotemFullscreen *fs)
 {
-	fs->priv->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
-						 (GSourceFunc) totem_fullscreen_popup_hide, fs);
+	fs->priv->popup_timeout = g_timeout_add_seconds (FULLSCREEN_POPUP_TIMEOUT,
+							 (GSourceFunc) totem_fullscreen_popup_hide, fs);
 }
 
 static void
@@ -216,7 +198,39 @@ totem_fullscreen_popup_timeout_remove (TotemFullscreen *fs)
 static void
 totem_fullscreen_set_cursor (TotemFullscreen *fs, gboolean state)
 {
-	bacon_video_widget_set_show_cursor (fs->priv->bvw, state);
+	if (fs->priv->bvw != NULL)
+		bacon_video_widget_set_show_cursor (fs->priv->bvw, state);
+}
+
+static GtkWidget *
+totem_fullscreen_get_volume_popup (TotemFullscreen *fs)
+{
+	return gtk_widget_get_toplevel (GTK_SCALE_BUTTON (fs->volume)->plus_button);
+}
+
+static gboolean
+totem_fullscreen_is_volume_popup_visible (TotemFullscreen *fs)
+{
+	GtkWidget *toplevel;
+
+	/* FIXME we should use the popup-visible property instead */
+	toplevel = totem_fullscreen_get_volume_popup (fs);
+	return GTK_WIDGET_VISIBLE (toplevel);
+}
+
+static void
+totem_fullscreen_force_popup_hide (TotemFullscreen *fs)
+{
+	/* Popdown the volume button if it's visible */
+	if (totem_fullscreen_is_volume_popup_visible (fs))
+		gtk_bindings_activate (GTK_OBJECT (fs->volume), GDK_Escape, 0);
+
+	gtk_widget_hide (fs->priv->exit_popup);
+	gtk_widget_hide (fs->priv->control_popup);
+
+	totem_fullscreen_popup_timeout_remove (fs);
+
+	totem_fullscreen_set_cursor (fs, FALSE);
 }
 
 static gboolean
@@ -225,15 +239,10 @@ totem_fullscreen_popup_hide (TotemFullscreen *fs)
 	if (fs->priv->bvw == NULL || totem_fullscreen_is_fullscreen (fs) == FALSE)
 		return TRUE;
 
-	if (fs->priv->seek_lock != FALSE || fs->priv->vol_lock != FALSE)
+	if (fs->priv->seek_lock != FALSE || totem_fullscreen_is_volume_popup_visible (fs) != FALSE)
 		return TRUE;
 
-	gtk_widget_hide (fs->priv->exit_popup);
-	gtk_widget_hide (fs->priv->control_popup);
-
-	totem_fullscreen_popup_timeout_remove (fs);
-
-	totem_fullscreen_set_cursor (fs, FALSE);
+	totem_fullscreen_force_popup_hide (fs);
 
 	return FALSE;
 }
@@ -242,7 +251,7 @@ gboolean
 totem_fullscreen_motion_notify (GtkWidget *widget, GdkEventMotion *event,
 				TotemFullscreen *fs)
 {
-	GtkWidget * item;
+	GtkWidget *item;
 
 	if (totem_fullscreen_is_fullscreen (fs) == FALSE) 
 		return FALSE;
@@ -250,6 +259,9 @@ totem_fullscreen_motion_notify (GtkWidget *widget, GdkEventMotion *event,
 	if (fs->priv->popup_in_progress != FALSE)
 		return FALSE;
 
+	if (gtk_window_is_active (GTK_WINDOW (fs->priv->parent_window)) == FALSE)
+		return FALSE;
+
 	fs->priv->popup_in_progress = TRUE;
 
 	totem_fullscreen_popup_timeout_remove (fs);
@@ -282,7 +294,7 @@ totem_fullscreen_set_fullscreen (TotemFullscreen *fs,
 {
 	g_return_if_fail (TOTEM_IS_FULLSCREEN (fs));
 
-	totem_fullscreen_popup_hide (fs);
+	totem_fullscreen_force_popup_hide (fs);
 
 	bacon_video_widget_set_fullscreen (fs->priv->bvw, fullscreen);
 	totem_fullscreen_set_cursor (fs, !fullscreen);
@@ -290,6 +302,27 @@ totem_fullscreen_set_fullscreen (TotemFullscreen *fs,
 	fs->is_fullscreen = fullscreen;
 }
 
+static void
+totem_fullscreen_parent_window_notify (GtkWidget *parent_window,
+				       GParamSpec *property,
+				       TotemFullscreen *fs)
+{
+	GtkWidget *popup;
+
+	if (totem_fullscreen_is_fullscreen (fs) == FALSE)
+		return;
+
+	popup = totem_fullscreen_get_volume_popup (fs);
+	if (parent_window == fs->priv->parent_window &&
+	    gtk_window_is_active (GTK_WINDOW (parent_window)) == FALSE &&
+	    GTK_WIDGET_VISIBLE (popup) == FALSE) {
+		totem_fullscreen_force_popup_hide (fs);
+		totem_fullscreen_set_cursor (fs, TRUE);
+	} else {
+		totem_fullscreen_set_cursor (fs, FALSE);
+	}
+}
+
 TotemFullscreen *
 totem_fullscreen_new (GtkWindow *toplevel_window)
 {
@@ -300,7 +333,7 @@ totem_fullscreen_new (GtkWindow *toplevel_window)
 
 
 	priv = fs->priv;
-	priv->vol_lock = priv->seek_lock = FALSE;
+	priv->seek_lock = FALSE;
 	priv->xml = totem_interface_load ("fullscreen.ui", TRUE, NULL, fs);
 
 	priv->exit_popup = GTK_WIDGET (gtk_builder_get_object (priv->xml,
@@ -321,8 +354,8 @@ totem_fullscreen_new (GtkWindow *toplevel_window)
 			  G_CALLBACK (totem_fullscreen_window_realize_cb), fs);
 	g_signal_connect (fs->priv->parent_window, "unrealize",
 			  G_CALLBACK (totem_fullscreen_window_unrealize_cb), fs);
-	g_signal_connect_swapped (G_OBJECT (fs->priv->parent_window), "notify::is-active",
-				  G_CALLBACK (totem_fullscreen_popup_hide), fs);
+	g_signal_connect (G_OBJECT (fs->priv->parent_window), "notify::is-active",
+			  G_CALLBACK (totem_fullscreen_parent_window_notify), fs);
 
 	/* Volume */
 	fs->volume = GTK_WIDGET (gtk_builder_get_object (priv->xml, "tcw_volume_button"));
