@@ -60,10 +60,10 @@ extern void totemobject_marshal_VOID__STRING_STRING_STRING (GClosure     *closur
                                                             gpointer      marshal_data);
 void
 totemobject_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
-                                                GValue       *return_value,
+                                                GValue       *return_value G_GNUC_UNUSED,
                                                 guint         n_param_values,
                                                 const GValue *param_values,
-                                                gpointer      invocation_hint,
+                                                gpointer      invocation_hint G_GNUC_UNUSED,
                                                 gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING) (gpointer     data1,
